"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseEpub = parseEpub;
const path_1 = require("path");
const utils_1 = require("./utils");
function parseSectionChildren(children) {
    var _a, _b, _c, _d, _e, _f, _g;
    const results = [];
    switch (children['#name']) {
        case 'div':
        case 'section':
        case 'nav':
        case 'ol':
        case 'ul':
            if ((_a = children.$$) === null || _a === void 0 ? void 0 : _a.length) {
                let extened_text = '';
                for (const child of children.$$) {
                    for (const el of parseSectionChildren(child)) {
                        switch (el.tag) {
                            case 'div':
                            case 'section':
                            case 'nav':
                            case 'ol':
                            case 'ul':
                            case 'li':
                            case 'img':
                            case 'image':
                            case 'svg':
                            case 'p':
                            case 'h1':
                            case 'h2':
                            case 'h3':
                            case 'h4':
                            case 'h5':
                            case 'h6':
                                if (extened_text.length) {
                                    results.push({
                                        type: 'text',
                                        value: extened_text,
                                        tag: 'div',
                                    });
                                    extened_text = '';
                                }
                                results.push(el);
                                break;
                            default:
                                if ((_b = el.value) === null || _b === void 0 ? void 0 : _b.length) {
                                    extened_text += el.value;
                                }
                                break;
                        }
                    }
                }
                if (extened_text.length) {
                    results.push({
                        type: 'text',
                        value: extened_text,
                        tag: children['#name'],
                    });
                }
            }
            else if ((_c = children._) === null || _c === void 0 ? void 0 : _c.length) {
                results.push({
                    type: 'text',
                    value: children._,
                    tag: children['#name'],
                });
            }
            break;
        case 'p':
        case 'a':
        case 'li':
        case 'h1':
        case 'h2':
        case 'h3':
        case 'h4':
        case 'h5':
        case 'h6':
            if ((_d = children.$$) === null || _d === void 0 ? void 0 : _d.length) {
                let extened_text = '';
                for (const child of children.$$) {
                    for (const el of parseSectionChildren(child)) {
                        if (el.type === 'image') {
                            results.push(el);
                        }
                        else if ((_e = el.value) === null || _e === void 0 ? void 0 : _e.length) {
                            extened_text += el.value;
                        }
                    }
                }
                if (extened_text.length) {
                    results.push({
                        type: 'text',
                        value: extened_text,
                        tag: children['#name'],
                    });
                }
            }
            else if ((_f = children._) === null || _f === void 0 ? void 0 : _f.length) {
                results.push({
                    type: 'text',
                    value: children._,
                    tag: children['#name'],
                });
            }
            break;
        case 'svg':
            for (const child of children.$$) {
                results.push(...parseSectionChildren(child));
            }
            break;
        case 'image':
        case 'img':
            const value = children.$.src || children.$['xlink:href'];
            if (value.length) {
                results.push({
                    type: 'image',
                    value: value,
                    tag: 'img',
                });
            }
            break;
        default:
            if ((_g = children._) === null || _g === void 0 ? void 0 : _g.length) {
                results.push({
                    type: 'text',
                    value: children._,
                    tag: children['#name'],
                });
            }
            break;
    }
    return results.flat(Infinity);
}
async function parseEpub(book) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    const files = await (0, utils_1.unzip)(book);
    const parsed_book = {};
    const container_file = files.find((f) => f.path.includes('container.xml'));
    const container_file_json = await (0, utils_1.convertXmlToJson)(container_file.data);
    const rootfile_path = container_file_json.container.rootfiles[0].rootfile[0].$['full-path'];
    const rootfile = files.find((f) => f.path.includes((0, path_1.basename)(rootfile_path)));
    const rootfile_json = await (0, utils_1.convertXmlToJson)(rootfile.data);
    const manifest = rootfile_json.package.manifest[0];
    let metadata;
    if (rootfile_json.package.metadata != null) {
        metadata = rootfile_json.package.metadata[0];
    }
    else if (rootfile_json.package['opf:metadata'] != null) {
        metadata = rootfile_json.package['opf:metadata'][0];
    }
    const sections = [];
    const images = [];
    const styles = [];
    const other = [];
    for (const item of manifest.item) {
        if (['application/xhtml+xml'].includes(item.$['media-type'])) {
            sections.push(item.$);
        }
        else if (['image/jpeg', 'image/png'].includes(item.$['media-type'])) {
            images.push(item.$);
        }
        else if (['text/css'].includes(item.$['media-type'])) {
            styles.push(item.$);
        }
        else {
            other.push(item.$);
        }
    }
    parsed_book.title = (_a = metadata['dc:title']) === null || _a === void 0 ? void 0 : _a[0];
    parsed_book.lang = (_b = metadata['dc:language']) === null || _b === void 0 ? void 0 : _b[0];
    parsed_book.author = (_d = (_c = metadata['dc:creator']) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d._;
    parsed_book.publisher = (_e = metadata['dc:publisher']) === null || _e === void 0 ? void 0 : _e[0];
    let cover_from_meta;
    if (metadata.meta) {
        cover_from_meta = (_g = (_f = metadata.meta.find((m) => m.$.name === 'cover')) === null || _f === void 0 ? void 0 : _f.$) === null || _g === void 0 ? void 0 : _g.content;
    }
    if (!cover_from_meta && metadata['opf:meta']) {
        cover_from_meta = (_j = (_h = metadata['opf:meta'].find((m) => m.$.name === 'cover')) === null || _h === void 0 ? void 0 : _h.$) === null || _j === void 0 ? void 0 : _j.content;
    }
    if (cover_from_meta) {
        const cover_image = images.find((i) => i.id === cover_from_meta);
        const cover_file = files.find((f) => f.path.includes((0, path_1.basename)(cover_image.href)));
        if (cover_file) {
            parsed_book.cover = {
                id: cover_from_meta,
                href: cover_image.href,
                'media-type': 'image/jpeg',
                parsed_data: [
                    {
                        type: 'image',
                        value: cover_image.href,
                        base64: (0, utils_1.toBase64)(cover_file.data, 'image/jpeg'),
                        tag: 'img',
                    },
                ],
            };
        }
    }
    parsed_book.sections = sections;
    for (const section of parsed_book.sections) {
        const section_file = files.find((f) => f.path.includes((0, path_1.basename)(section.href)));
        const section_json = await (0, utils_1.convertXmlToJson)(section_file.data, {
            preserveChildrenOrder: true,
            charsAsChildren: true,
            explicitChildren: true,
        });
        const section_data = [];
        for (const body of section_json.html.body) {
            for (const tag of body.$$) {
                for (const el of parseSectionChildren(tag)) {
                    if ((_k = el.value) === null || _k === void 0 ? void 0 : _k.length) {
                        if (el.type === 'image') {
                            const image_file = files.find((f) => f.path.includes((0, path_1.basename)(el.value)));
                            if (image_file) {
                                el.base64 = (0, utils_1.toBase64)(image_file.data, 'image/jpeg');
                            }
                        }
                        section_data.push(el);
                    }
                }
            }
        }
        section.parsed_data = section_data;
    }
    return parsed_book;
}
//# sourceMappingURL=index.js.map